package io.tswf.minecraft.blockventure.modloader.init

import groovy.transform.CompileStatic
import groovy.util.logging.Slf4j
import io.tswf.minecraft.blockventure.modloader.api.IClientSetupAware
import io.tswf.minecraft.blockventure.modloader.api.ICommonSetupAware
import io.tswf.minecraft.blockventure.modloader.exception.InvalidModDescriptionException
import io.tswf.minecraft.blockventure.modloader.meta.ModMetadata
import io.tswf.minecraft.blockventure.modloader.meta.ModStatus
import io.tswf.minecraft.blockventure.modloader.scanner.ModsScanner
import io.tswf.minecraft.metamorphosis.core.side.SidesResolver

@Slf4j(category = "BlockVenture")
@CompileStatic
class BlockVentureModLoader {
    static List<ModMetadata> modsMeta = new ArrayList<>()

    static void scanMods() {
        log.info("BlockVentureModsInit.scanMods")
        modsMeta = ModsScanner.scanModsFromClasspath()
    }

    static void createMods() {
        log.info("BlockVentureModsInit.createMods")
        for (def modMeta : modsMeta) {
            def modObjectClass = ClassLoader.contextClassloader().loadClass(modMeta.runtimeMeta.modClassNode.javaName)
            modMeta.runtimeMeta.modClass = modObjectClass

            def modInstance = modObjectClass.noArgsConstructor
                .ensureNotNull { new InvalidModDescriptionException("There is no no-args constructor found in class '${modObjectClass.name}'!", modMeta) }
                .newInstance()

            modMeta.runtimeMeta.modInstance = modInstance
            modMeta.runtimeMeta.status = ModStatus.CREATED
        }
    }

    static void initMods() {
        log.info("BlockVentureModsInit.createMods")

        for (def modMeta : modsMeta) {
            def modInstance = modMeta.runtimeMeta.modInstance
            if (modInstance instanceof ICommonSetupAware) {
                modInstance.setupCommon()
            }

            if (SidesResolver.isClientPresented()) {
                if (modInstance instanceof IClientSetupAware) {
                    modInstance.setupClient()
                }
            }

            modMeta.runtimeMeta.status = ModStatus.INITIALIZED
        }
    }
}
